@@ -20,13 +20,16 @@ use strict;
 use warnings;
 
 our $VERSION = '0.01';
-
-use Fcntl qw(:flock);
-use Dpkg::Gettext;
-use Dpkg::ErrorHandling;
+our @EXPORT = qw(
+    file_lock
+    file_slurp
+);
 
 use Exporter qw(import);
-our @EXPORT = qw(file_lock file_slurp);
+use Fcntl qw(:flock);
+
+use Dpkg::Gettext;
+use Dpkg::ErrorHandling;
 
 sub file_lock($$) {
     my ($fh, $filename) = @_;
@@ -35,22 +38,25 @@ sub file_lock($$) {
     # and dpkg-dev indirectly making use of it, makes building new perl
     # package which bump the perl ABI impossible as these packages cannot
     # be installed alongside.
-    eval 'use File::FcntlLock';
+    eval q{
+        pop @INC if $INC[-1] eq '.';
+        use File::FcntlLock;
+    };
     if ($@) {
-        warning(_g('File::FcntlLock not available; using flock which is not NFS-safe'));
+        warning(g_('File::FcntlLock not available; using flock which is not NFS-safe'));
         flock($fh, LOCK_EX)
-            or syserr(_g('failed to get a write lock on %s'), $filename);
+            or syserr(g_('failed to get a write lock on %s'), $filename);
     } else {
         eval q{
             my $fs = File::FcntlLock->new(l_type => F_WRLCK);
             $fs->lock($fh, F_SETLKW)
-                or syserr(_g('failed to get a write lock on %s'), $filename);
+                or syserr(g_('failed to get a write lock on %s'), $filename);
         }
     }
 }
 
 sub file_slurp {
-    my ($fh) = @_;
+    my $fh = shift;
 
     local $/;
     my $data = <$fh>;
