@@ -95,7 +95,6 @@ sub usage {
       --hook-<name>=<command> set <command> as the hook <name>, known hooks:
                                 init preclean source build binary buildinfo
                                 changes postclean check sign done
-      --buildinfo-id=<id>     set the <id> part of the .buildinfo filename.
       --buildinfo-option=<opt>
                               pass option <opt> to dpkg-genbuildinfo.
   -p, --sign-command=<command>
@@ -166,13 +165,12 @@ my $host_type = '';
 my $target_arch = '';
 my $target_type = '';
 my @build_profiles = ();
-my $call_target = '';
+my @call_target = ();
 my $call_target_as_root = 0;
 my $since;
 my $maint;
 my $changedby;
 my $desc;
-my $buildinfo_id;
 my @buildinfo_opts;
 my @changes_opts;
 my @hook_names = qw(
@@ -235,8 +233,9 @@ while (@ARGV) {
 	usageerr(g_('missing hook %s command'), $hook_name)
 	    if not defined $hook_cmd;
 	$hook{$hook_name} = $hook_cmd;
-    } elsif (/^--buildinfo-id=(.*)$/) {
-	$buildinfo_id = $1;
+    } elsif (/^--buildinfo-id=.*$/) {
+	# Deprecated option
+	warning('--buildinfo-id is deprecated, it is without effect');
     } elsif (/^(?:-p|--sign-command=)(.*)$/) {
 	$signcommand = $1;
     } elsif (/^(?:-k|--sign-key=)(.*)$/) {
@@ -288,9 +287,10 @@ while (@ARGV) {
     } elsif (/^--target-type=(.*)$/) {
 	$target_type = $1;
     } elsif (/^(?:--target|--rules-target|-T)$/) {
-        $call_target = shift @ARGV;
+        push @call_target, split /,/, shift @ARGV;
     } elsif (/^(?:--target=|--rules-target=|-T)(.+)$/) {
-        $call_target = $1;
+        my $arg = $1;
+        push @call_target, split /,/, $arg;
     } elsif (/^--as-root$/) {
         $call_target_as_root = 1;
     } elsif (/^--pre-clean$/) {
@@ -394,6 +394,9 @@ if (defined $parallel) {
         $parallel = qx(getconf _NPROCESSORS_ONLN 2>/dev/null);
         # Fallback for at least Irix.
         $parallel = qx(getconf _NPROC_ONLN 2>/dev/null) if $?;
+        # Fallback to serial execution if cannot infer the number of online
+        # processors.
+        $parallel = '1' if $?;
         chomp $parallel;
     }
     if ($parallel_force) {
@@ -404,14 +407,6 @@ if (defined $parallel) {
     $build_opts->export();
 }
 
-if (defined $buildinfo_id) {
-    # The .buildinfo identifiers have the same restrictions as package names.
-    my $err = pkg_name_is_illegal($buildinfo_id);
-    if ($err) {
-        error(g_("illegal .buildinfo ID '%s': %s"), $buildinfo_id, $err);
-    }
-}
-
 set_build_profiles(@build_profiles) if @build_profiles;
 
 my $cwd = cwd();
@@ -496,7 +491,7 @@ if (not -x 'debian/rules') {
     chmod(0755, 'debian/rules'); # No checks of failures, non fatal
 }
 
-unless ($call_target) {
+if (scalar @call_target == 0) {
     chdir('..') or syserr('chdir ..');
     withecho('dpkg-source', @source_opts, '--before-build', $dir);
     chdir($dir) or syserr("chdir $dir");
@@ -520,7 +515,7 @@ if ($checkbuilddep) {
     }
 }
 
-if ($call_target) {
+foreach my $call_target (@call_target) {
     if ($call_target_as_root or
         $call_target =~ /^(clean|binary(|-arch|-indep))$/)
     {
@@ -528,8 +523,8 @@ if ($call_target) {
     } else {
         withecho(@debian_rules, $call_target);
     }
-    exit 0;
 }
+exit 0 if scalar @call_target;
 
 run_hook('preclean', ! $noclean);
 
@@ -564,7 +559,6 @@ if (build_has_any(BUILD_BINARY)) {
 run_hook('buildinfo', 1);
 
 push @buildinfo_opts, "--build=$build_types" if build_has_none(BUILD_DEFAULT);
-push @buildinfo_opts, "--buildinfo-id=$buildinfo_id" if $buildinfo_id;
 push @buildinfo_opts, "--admindir=$admindir" if $admindir;
 
 withecho('dpkg-genbuildinfo', @buildinfo_opts);
